#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    [Description("The Dinapoli Stochastic is simply an EMA smoothed stochastic, instead of having a SMA as smoothing algorithm")]
    [Gui.Design.DisplayName("Dinapoli Stochastic")]
    public class DinapoliStochastic : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int period = 14; // Default setting for Period
            private int lineK_Period = 6; // Default setting for LineK_Period
            private int lineD_Period = 6; // Default setting for LineD_Period
        // User defined variables (add any user defined variables below)
			private double lo = 0;
			private double dif = 0;
			private int alpha = 50;
			private bool extremes = true;
			private DataSeries k;
			private DataSeries Ktemp;
			private DataSeries Dtemp;
		#endregion
        protected override void Initialize()
        {
            Add(new Plot(Color.Orange, PlotStyle.Line, "LineK"));
            Add(new Plot(Color.Green, PlotStyle.Line, "LineD"));
            Add(new Line(Color.DarkOliveGreen, 80, "HiBase"));
            Add(new Line(Color.DarkOliveGreen, 20, "LoBase"));
            Period = 14;
            LineK_Period = 6;
            LineD_Period = 6;
			alpha = 50;
			CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= true;
			k = new DataSeries(this);
			Ktemp = new DataSeries(this);
			Dtemp = new DataSeries(this);
			}
        protected override void OnBarUpdate()
        {
			lo = MIN(Low, Period)[0];
			dif = MAX(High, Period)[0] - MIN(Low, Period)[0];
			k.Set(100*(Input[0]-lo)/dif);
			Ktemp.Set(EMA(k, LineK_Period)[0]);
			Dtemp.Set(EMA(Ktemp, LineD_Period)[0]);
			LineK.Set(Ktemp[0]);
            LineD.Set(Dtemp[0]);
			switch (extremes) {
				case false:
				if (Ktemp[0] > Dtemp[0]) 
   		     	{ 
        	        BackColor = Color.FromArgb(alpha,Color.Green); 
            	}
				else if (Ktemp[0] < Dtemp[0])
        		{ 
                	BackColor = Color.FromArgb(alpha,Color.Red); 
            	}
					break;
				case true:
				if (Ktemp[0] > Dtemp[0] && Dtemp[0] > 20 && Dtemp[0] < 80) 
    	    	{ 
        	        BackColor = Color.FromArgb(alpha,Color.Green); 
            	}
				else if (Dtemp[0] > 80)
				{
					BackColor = Color.FromArgb(alpha,Color.Green);
				}
				else if (Ktemp[0] < Dtemp[0] && Dtemp[0] < 80 && Dtemp[0] > 20)
				{
					BackColor = Color.FromArgb(alpha,Color.Red);
				}
				else if (Dtemp[0] < 20)
				{
					BackColor = Color.FromArgb(alpha,Color.Red);
				}
					
					break;
				default:
					
					break;
			}
		}

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LineK
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LineD
        {
            get { return Values[1]; }
        }

        [Description("Lookback Period")]
        [Category("Parameters")]
        public int Period
        {
            get { return period; }
            set { period = Math.Max(1, value); }
        }

        [Description("K Parameter smoothing")]
        [Category("Parameters")]
        public int LineK_Period
        {
            get { return lineK_Period; }
            set { lineK_Period = Math.Max(1, value); }
        }

        [Description("D Parameter smoothing")]
        [Category("Parameters")]
        public int LineD_Period
        {
            get { return lineD_Period; }
            set { lineD_Period = Math.Max(1, value); }
        }

		[Description("Opacity of color background")]
        [Category("General")]
        public int Alpha
        {
            get { return alpha; }
            set { alpha = Math.Max(1, value); }
        }

        [Description("Changes color behavior to either signals or the extremes (20,80)")]
        [Category("General")]
		public bool Extremes
        {
            get { return extremes; }
            set { extremes = value; }
        }

        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private DinapoliStochastic[] cacheDinapoliStochastic = null;

        private static DinapoliStochastic checkDinapoliStochastic = new DinapoliStochastic();

        /// <summary>
        /// The Dinapoli Stochastic is simply an EMA smoothed stochastic, instead of having a SMA as smoothing algorithm
        /// </summary>
        /// <returns></returns>
        public DinapoliStochastic DinapoliStochastic(int lineD_Period, int lineK_Period, int period)
        {
            return DinapoliStochastic(Input, lineD_Period, lineK_Period, period);
        }

        /// <summary>
        /// The Dinapoli Stochastic is simply an EMA smoothed stochastic, instead of having a SMA as smoothing algorithm
        /// </summary>
        /// <returns></returns>
        public DinapoliStochastic DinapoliStochastic(Data.IDataSeries input, int lineD_Period, int lineK_Period, int period)
        {
            checkDinapoliStochastic.LineD_Period = lineD_Period;
            lineD_Period = checkDinapoliStochastic.LineD_Period;
            checkDinapoliStochastic.LineK_Period = lineK_Period;
            lineK_Period = checkDinapoliStochastic.LineK_Period;
            checkDinapoliStochastic.Period = period;
            period = checkDinapoliStochastic.Period;

            if (cacheDinapoliStochastic != null)
                for (int idx = 0; idx < cacheDinapoliStochastic.Length; idx++)
                    if (cacheDinapoliStochastic[idx].LineD_Period == lineD_Period && cacheDinapoliStochastic[idx].LineK_Period == lineK_Period && cacheDinapoliStochastic[idx].Period == period && cacheDinapoliStochastic[idx].EqualsInput(input))
                        return cacheDinapoliStochastic[idx];

            DinapoliStochastic indicator = new DinapoliStochastic();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.LineD_Period = lineD_Period;
            indicator.LineK_Period = lineK_Period;
            indicator.Period = period;
            indicator.SetUp();

            DinapoliStochastic[] tmp = new DinapoliStochastic[cacheDinapoliStochastic == null ? 1 : cacheDinapoliStochastic.Length + 1];
            if (cacheDinapoliStochastic != null)
                cacheDinapoliStochastic.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheDinapoliStochastic = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The Dinapoli Stochastic is simply an EMA smoothed stochastic, instead of having a SMA as smoothing algorithm
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DinapoliStochastic DinapoliStochastic(int lineD_Period, int lineK_Period, int period)
        {
            return _indicator.DinapoliStochastic(Input, lineD_Period, lineK_Period, period);
        }

        /// <summary>
        /// The Dinapoli Stochastic is simply an EMA smoothed stochastic, instead of having a SMA as smoothing algorithm
        /// </summary>
        /// <returns></returns>
        public Indicator.DinapoliStochastic DinapoliStochastic(Data.IDataSeries input, int lineD_Period, int lineK_Period, int period)
        {
            return _indicator.DinapoliStochastic(input, lineD_Period, lineK_Period, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The Dinapoli Stochastic is simply an EMA smoothed stochastic, instead of having a SMA as smoothing algorithm
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DinapoliStochastic DinapoliStochastic(int lineD_Period, int lineK_Period, int period)
        {
            return _indicator.DinapoliStochastic(Input, lineD_Period, lineK_Period, period);
        }

        /// <summary>
        /// The Dinapoli Stochastic is simply an EMA smoothed stochastic, instead of having a SMA as smoothing algorithm
        /// </summary>
        /// <returns></returns>
        public Indicator.DinapoliStochastic DinapoliStochastic(Data.IDataSeries input, int lineD_Period, int lineK_Period, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.DinapoliStochastic(input, lineD_Period, lineK_Period, period);
        }

    }
}
#endregion
